import compas
import compas_slicer

if __name__ == '__main__':
    print('COMPAS: {}'.format(compas.__version__))
    print('COMPAS Slicer: {}'.format(compas_slicer.__version__))
    print('Awesome! Your installation worked! :)')
